mirror of https://lore.kernel.org/lkml/
 help / color / mirror / Atom feed
From: Jim Cromie <jim.cromie@gmail.com>
To: jbaron@redhat.com
Cc: linux-kernel@vger.kernel.org, bvanassche@acm.org,
	joe@perches.com, gregkh@suse.de, gnb@fmeh.org,
	Jim Cromie <jim.cromie@gmail.com>
Subject: [PATCH 13/21] dynamic_debug: require 'a' flag to explicitly mark pending queries
Date: Mon, 11 Jul 2011 01:46:48 -0600	[thread overview]
Message-ID: <1310370416-6322-14-git-send-email-jim.cromie@gmail.com> (raw)
In-Reply-To: <1310370416-6322-1-git-send-email-jim.cromie@gmail.com>

Require that user mark pending queries explicitly, otherwise warn
that they do not apply.  Note that pending rules are still applied
by default, and are only added to pending-list if they do not apply,
and bear the flag.

Also test that query hasnt already been added to pending-list,
if it has (according to queries_match), just update the found query.

Signed-off-by: Jim Cromie <jim.cromie@gmail.com>
---
 lib/dynamic_debug.c |   52 +++++++++++++++++++++++++++++++++++++++++++++-----
 1 files changed, 46 insertions(+), 6 deletions(-)

diff --git a/lib/dynamic_debug.c b/lib/dynamic_debug.c
index bee1376..0faac83 100644
--- a/lib/dynamic_debug.c
+++ b/lib/dynamic_debug.c
@@ -91,6 +91,7 @@ static struct { unsigned flag:8; char opt_char; } opt_array[] = {
 	{ _DPRINTK_FLAGS_INCL_FUNCNAME, 'f' },
 	{ _DPRINTK_FLAGS_INCL_LINENO, 'l' },
 	{ _DPRINTK_FLAGS_INCL_TID, 't' },
+	{ _DPRINTK_FLAGS_APPEND, 'a' },
 };
 
 /* format a string into buf[] which describes the _ddebug's flags */
@@ -454,12 +455,47 @@ static int ddebug_parse_flags(const char *str, unsigned int *flagsp,
 	return 0;
 }
 
-/* copy query off stack, save flags & mask, and store in pending-list */
+static int queries_match(struct ddebug_query *q1, struct ddebug_query *q2)
+{
+	if (!q1->module ^ !q2->module ||
+		!q1->filename ^ !q2->filename ||
+		!q1->function ^ !q2->function ||
+		!q1->format ^ !q2->format)
+		return 0;  /* a match-spec set/unset state differs */
+
+	if (q1->last_lineno != q2->last_lineno ||
+		q1->first_lineno != q2->first_lineno)
+		return 0;
+
+	if ((q1->module && strcmp(q1->module, q2->module)) ||
+		(q1->filename && strcmp(q1->filename, q2->filename)) ||
+		(q1->function && strcmp(q1->function, q2->function)) ||
+		(q1->format && strcmp(q1->format, q2->format)))
+		return 0;
+	return 1;
+}
+
+/* copy query off stack, save flags & mask, and store in pending-list,
+   after checking that query isnt already there
+ */
 static int ddebug_save_pending(struct ddebug_query *query,
 				unsigned int flags, unsigned int mask)
 {
-	struct pending_query *pq;
+	struct pending_query *pq, *pqnext;
 
+	list_for_each_entry_safe(pq, pqnext, &pending_queries, link) {
+		if (queries_match(query, &pq->query)) {
+			/* query already in list, update flags */
+			if (pq->flags != flags)
+				pq->flags = flags;
+			if (pq->mask != mask)
+				pq->mask = mask;
+			if (verbose)
+				pr_info("already pending, updated it: %s\n",
+					show_pending_query(pq));
+			return 0;
+		}
+	}
 	if (verbose)
 		pr_info("add to pending: %s\n", show_ddebug_query(query));
 
@@ -511,9 +547,13 @@ static int ddebug_exec_query(char *query_string)
 	nfound = ddebug_change(&query, flags, mask);
 
 	pr_info("nfound %d on %s\n", nfound, show_ddebug_query(&query));
-	if (!nfound)
-		return ddebug_save_pending(&query, flags, mask);
-
+	if (!nfound) {
+		if (flags & _DPRINTK_FLAGS_APPEND)
+			return ddebug_save_pending(&query, flags, mask);
+		else
+			pr_warn("no match on: %s\n",
+				show_ddebug_query(&query));
+	}
 	return 0;
 }
 
-- 
1.7.4.1


  parent reply	other threads:[~2011-07-11  7:49 UTC|newest]

Thread overview: 72+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2011-06-28  7:09 [PATCH 0/11] dynamic_debug: allow multiple pending queries on boot-line Jim Cromie
2011-06-28  7:09 ` [PATCH 01/11] dynamic_debug: allow changing of dynamic_debug verbosity any time Jim Cromie
2011-06-29 10:41   ` Bart Van Assche
2011-07-01 21:57     ` Greg KH
2011-07-02  8:39       ` Jim Cromie
2011-06-28  7:09 ` [PATCH 02/11] dynamic_debug: trim source-path prefix from dynamic_debug/control Jim Cromie
2011-06-28 10:08   ` Bart Van Assche
2011-06-28 18:29     ` Jim Cromie
2011-06-28  7:09 ` [PATCH 03/11] dynamic_debug: process multiple commands on a line Jim Cromie
2011-06-29 10:50   ` Bart Van Assche
2011-07-05 16:12     ` Jim Cromie
2011-06-28  7:09 ` [PATCH 04/11] dynamic_debug: warn when >1 of each type of match-spec is given Jim Cromie
2011-06-28 10:17   ` Bart Van Assche
2011-06-28 15:21     ` Jim Cromie
2011-06-28  7:09 ` [PATCH 05/11] dynamic_debug: use pr_info instead of printk(KERN_INFO Jim Cromie
2011-06-28  7:09 ` [PATCH 06/11] dynamic_debug: KERN_ERR should not depend upon verbosity Jim Cromie
2011-06-28  7:09 ` [PATCH 07/11] dynamic_debug: dont kill entire facility on error parsing ddebug_query Jim Cromie
2011-06-28  7:09 ` [PATCH 08/11] dynamic_debug: return int from ddebug_change Jim Cromie
2011-06-28 10:24   ` Bart Van Assche
2011-06-28 14:54     ` Jason Baron
2011-06-28 17:48       ` Bart Van Assche
2011-06-28 18:24         ` Jim Cromie
2011-06-28  7:09 ` [PATCH 09/11] dynamic_debug: add_to_pending() saves non-matching queries for later Jim Cromie
2011-06-28 14:48   ` Jason Baron
2011-07-03  8:27     ` Bart Van Assche
2011-06-28 19:17   ` Jim Cromie
2011-07-03  8:24     ` Bart Van Assche
2011-07-03  8:32   ` Bart Van Assche
2011-06-28  7:09 ` [PATCH 10/11] dynamic_debug: call apply_pending_queries from ddebug_add_module Jim Cromie
2011-07-03  8:37   ` Bart Van Assche
2011-06-28  7:09 ` [PATCH 11/11] dynamic_debug: document use of pendinq queries at boot-time Jim Cromie
2011-07-03  8:19 ` [PATCH 0/11] dynamic_debug: allow multiple pending queries on boot-line Bart Van Assche
2011-07-11  7:46 ` Jim Cromie
2011-07-11  7:46   ` [PATCH 01/21] dynamic_debug: add pending flag 'a' to make pending queries explicit Jim Cromie
2011-07-11  7:46   ` [PATCH 02/21] dynamic_debug: allow changing of dynamic_debug verbosity any time Jim Cromie
2011-07-11  7:46   ` [PATCH 03/21] dynamic_debug: process multiple commands on a line Jim Cromie
2011-07-12  5:54     ` Bart Van Assche
2011-07-13  7:25       ` Jim Cromie
2011-07-13 17:44         ` Bart Van Assche
2011-07-11  7:46   ` [PATCH 04/21] dynamic_debug: warn when >1 of each type of match-spec is given Jim Cromie
2011-07-11  7:46   ` [PATCH 05/21] dynamic_debug: pr_err() call should not depend upon verbosity Jim Cromie
2011-07-11  7:46   ` [PATCH 06/21] dynamic_debug: dont kill entire facility on error parsing ddebug_query Jim Cromie
2011-07-11  7:46   ` [PATCH 07/21] dynamic_debug: return int from ddebug_change Jim Cromie
2011-07-11 18:39     ` Bart Van Assche
2011-07-11  7:46   ` [PATCH 08/21] dynamic_debug: factor show_ddebug_query out of ddebug_parse_query Jim Cromie
2011-07-11 18:36     ` Bart Van Assche
2011-07-16 21:32       ` Jim Cromie
2011-07-25 11:10         ` Bart Van Assche
2011-07-25 13:56           ` Jim Cromie
2011-07-11  7:46   ` [PATCH 09/21] dynamic_debug: save_pending() saves non-matching queries for later Jim Cromie
2011-07-11 18:32     ` Bart Van Assche
2011-07-12  5:55     ` Bart Van Assche
2011-07-11  7:46   ` [PATCH 10/21] dynamic_debug: call apply_pending_queries from ddebug_add_module Jim Cromie
2011-07-11  7:46   ` [PATCH 11/21] dynamic_debug: refactor query_matches_callsite out of ddebug_change Jim Cromie
2011-07-11  7:46   ` [PATCH 12/21] dynamic_debug: document use of pending queries at boot-time Jim Cromie
2011-07-11  7:46   ` Jim Cromie [this message]
2011-07-12  9:20     ` [PATCH 13/21] dynamic_debug: require 'a' flag to explicitly mark pending queries Bart Van Assche
2011-07-11  7:46   ` [PATCH 14/21] dynamic_debug: hoist locking in ddebug_change to callers Jim Cromie
2011-07-11  7:46   ` [PATCH 15/21] dynamic_debug: describe_flags with '=[ptmfl]*' Jim Cromie
2011-07-11  7:46   ` [PATCH 16/21] dynamic_debug: define several levels of verbosity Jim Cromie
2011-07-11  7:46   ` [PATCH 17/21] dynamic_debug: non-optimization - remove != NULL Jim Cromie
2011-07-11  7:46   ` [PATCH 18/21] dynamic_debug: trim source-path prefix from dynamic_debug/control Jim Cromie
2011-07-11  7:46   ` [PATCH 19/21] dynamic_debug: add flags filtering to flags spec handling Jim Cromie
2011-07-11  7:46   ` [PATCH 20/21] dynamic_debug: clear pending_queries list in remove_all_tables Jim Cromie
2011-07-11  7:46   ` [PATCH 21/21] dynamic_debug: delete pending queries Jim Cromie
2011-07-11 10:49     ` Bart Van Assche
2011-07-11 13:43       ` Jim Cromie
2011-07-12  0:25     ` Joe Perches
2011-07-12 20:50       ` Jason Baron
2011-07-20  5:39         ` Jim Cromie
2011-07-20 17:43       ` Jim Cromie
2011-07-20 22:08         ` Joe Perches

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=1310370416-6322-14-git-send-email-jim.cromie@gmail.com \
    --to=jim.cromie@gmail.com \
    --cc=bvanassche@acm.org \
    --cc=gnb@fmeh.org \
    --cc=gregkh@suse.de \
    --cc=jbaron@redhat.com \
    --cc=joe@perches.com \
    --cc=linux-kernel@vger.kernel.org \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox

all inboxes | Powered by JetHome®